@@ -190,6 +190,8 @@ pdf_document_dispose (GObject *object)
 	if (pdf_document->fonts_iter) {
 		poppler_fonts_iter_free (pdf_document->fonts_iter);
 	}
+
+	G_OBJECT_CLASS (pdf_document_parent_class)->dispose (object);
 }
 
 static void
@@ -1250,8 +1252,8 @@ pdf_document_thumbnails_get_dimensions (EvDocumentThumbnails *document_thumbnail
 
 		poppler_page_get_size (poppler_page, &page_width, &page_height);
 
-		*width = (gint) (page_width * rc->scale);
-		*height = (gint) (page_height * rc->scale);
+		*width = (gint) MAX (page_width * rc->scale, 1);
+		*height = (gint) MAX (page_height * rc->scale, 1);
 	}
 
 	if (rc->rotation == 90 || rc->rotation == 270) {
@@ -1621,13 +1623,23 @@ pdf_selection_render_selection (EvSelection      *selection,
 					   TRUE, 8,
 					   width, height);
 	}
-	
+
+#if POPPLER_VERSION <= 5
 	poppler_page_render_selection (POPPLER_PAGE (rc->data),
 				       rc->scale, rc->rotation, *pixbuf,
 				       (PopplerRectangle *)points,
 				       (PopplerRectangle *)old_points,
 				       text,
 				       base);
+#else /* POPPLER_VERSION == 9 */
+	poppler_page_render_selection_to_pixbuf (POPPLER_PAGE (rc->data),
+						 rc->scale, rc->rotation, *pixbuf,
+						 (PopplerRectangle *)points,
+						 (PopplerRectangle *)old_points,
+						 POPPLER_SELECTION_NORMAL,
+						 text,
+						 base);
+#endif
 }
 
 
